// SPDX-License-Identifier: MIT

use askama::Template;
use axum::{
    http::StatusCode,
    response::{Html, IntoResponse, Response},
};

use crate::common::{self, Config, SideMenu};

pub struct RawTemplate<T>(pub T);

impl<T> IntoResponse for RawTemplate<T>
where
    T: Template,
{
    fn into_response(self) -> Response {
        match self.0.render() {
            Ok(html) => Html(html).into_response(),
            Err(err) => (
                StatusCode::INTERNAL_SERVER_ERROR,
                format!("Failed to render template. Error: {}", err),
            )
                .into_response(),
        }
    }
}

#[derive(Template)]
#[template(path = "../src/common/templates/common.html")]
pub struct HtmlTemplate<T: Template> {
    side_menu: Vec<SideMenu>,
    title: String,
    header_title: String,
    version: &'static str,
    contents: T,
}

impl<T: Template> HtmlTemplate<T> {
    pub fn new<S: Into<String>>(title: S, contents: T) -> HtmlTemplate<T> {
        let customize = &Config::get().customize.get();
        HtmlTemplate {
            side_menu: customize.side_menu.clone(),
            title: title.into(),
            header_title: customize.header_title.clone(),
            version: common::VERSION,
            contents,
        }
    }
}

impl<T: Template> IntoResponse for HtmlTemplate<T> {
    fn into_response(self) -> Response {
        match self.render() {
            Ok(html) => Html(html).into_response(),
            Err(err) => (
                StatusCode::INTERNAL_SERVER_ERROR,
                format!("Failed to render template. Error: {}", err),
            )
                .into_response(),
        }
    }
}
